package cscie259.project3.wahoo;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;

import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;


/**
 * This servlet logs the user into the Wahoo portal.
 *
 * You MAY modify this file.
 *
 * @author  Computer Science E-259
 * @version 6.0
 *
 * @author  YOUR NAME GOES HERE
 **/

public class Login extends WahooServlet
{
    /**
     * Presents user with login screen and processes login attempts
     * and account creations.
     *
     * @param request  HTTP request object
     * @param response HTTP response object
     *
     * @throws IOException      if a file-related error occurs
     * @throws ServletException if a servlet-related error occurs
     */
    protected void doWork(HttpServletRequest request,
                          HttpServletResponse response)
    throws IOException, ServletException
    {
        // check to see if a session already exists; if so, invalidate it
        HttpSession session = request.getSession(false);
        if (session != null)
        {
            // attempt to retrieve username from session
            String username = (String) session.getAttribute("username");

            // if username is found, log user out (by invalidating session)
            if (username != null)
                System.out.println("User " + username + " logged out");
            session.invalidate();

            // display the login screen
            displayLoginForm(request, response, null);

            return;
        }

        // process form submission, else display login screen
        String operation = request.getParameter("submit");
        if (operation != null &&
            (operation.equals("Log In") || operation.equals("Register")))
        {
            // retrieve username from HTTP request
            String username = request.getParameter("username");

            // if username missing, display the login screen with an
            // informative error message
            if (username == null || username.equals(""))
            {
                displayLoginForm(request, response, "Please enter a username");
                return;
            }

            // retrieve password from HTTP request
            String password = request.getParameter("password");

            // if password missing, display the login screen with an
            // informative error message
            if (password == null || password.equals(""))
            {
                displayLoginForm(request, response, "Please enter a password");
                return;
            }

            // retrieve User object with given username; if password incorrect,
            // display the login screen with an informative error message
            if (operation.equals("Log In"))
            {
                User user = UserManager.getUser(username);
                if (user == null || !user.getPassword().equals(password))
                {
                    displayLoginForm(request, response,
                                     "Invalid username or password");
                    return;
                }
            }
            else
            {
                // attempt create a new user, erring if desired username 
                // username is already in use
                User user = UserManager.addUser(username, password);
                if (user == null)
                {
                    displayLoginForm(request, response,
                                     "Unable to add user " + username + ".  " +
                                     "Please try a different username.");
                    return;
                }

                // save the new user!
                UserManager.save();
            }

            // report successful login
            System.out.println("User " + username + " logged in");

            // establish a new session and store the username in it
            session = request.getSession(true);
            session.setAttribute("username", username);

            // redirect user to view servlet
            redirect(response, "/servlet/view");

            return;
        }

        // else display the login screen by default
        else
        {
            displayLoginForm(request, response, null);
        }
    }


    /**
     * Displays the login form, with an optional error string.
     *
     * @param request  HTTP request object
     * @param response HTTP response object
     * @param error    optional error string
     *
     * @throws IOException      if an IO-related error occurs
     * @throws ServletException if a servlet-related error occurs
     */
    public void displayLoginForm(HttpServletRequest request,
                                 HttpServletResponse response,
                                 String error)
    throws IOException, ServletException
    {
        // output text/html MIME type
        response.setContentType("text/html");

        // retrieve PrintWriter
        PrintWriter out = response.getWriter();

        // attempt to apply login.xsl to dummy.xml
        try
        {
			// prepare to process login.xsl by way of TrAX
            TransformerFactory tFactory = TransformerFactory.newInstance();
            StreamSource xslSource =
            new StreamSource(getServletContext().getRealPath("/xsl/login.xsl"));
            Transformer transformer = tFactory.newTransformer(xslSource);

            // if there's an error message waiting to be displayed,
            // pass it to login.xsl as a stylesheet parameter
            if (error != null && !error.equals(""))
                transformer.setParameter("error", error);

            // apply login.xsl to dummy.xml by way of TrAX;
            // of course, a non-minimalist login screen might actually
            // make use of more interesting data (hint, hint)
            StreamSource xmlSource =
            new StreamSource(getServletContext().getRealPath("/xml/dummy.xml"));
            transformer.transform(xmlSource, new StreamResult(out));
        }

        // catch any exceptions
        catch (TransformerException e)
        {
             e.printStackTrace();
             throw new ServletException("Can't process login.xsl: " + 
                                        e.getMessage());
        }
    }
}
