package cscie259.project3.wahoo;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;

import java.util.HashMap;

import java.io.IOException;
import java.io.InputStream;

import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import javax.servlet.ServletContext;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import javax.xml.transform.OutputKeys;

import org.apache.xml.serializer.Serializer;
import org.apache.xml.serializer.SerializerFactory;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;


/**
 * This class keeps track of users of the Wahoo portal by writing out
 * information to an xml file.
 *
 * You MAY modify this file.
 *
 * @author  Computer Science E-259
 * @version 6.0
 *
 * @author  YOUR NAME GOES HERE
 **/

public final class UserManager
{
    /**
     * a reference to the one and only instance of this class
     */
    private static UserManager instance_;


    /**
     * a reference to the servlet context
     */
    private ServletContext context_;


    /**
     * a hash table of all users
     */
    private Map users_ = new HashMap();


    /**
     * A constructor that reads the user DB from an input stream.
     */
    private UserManager(ServletContext context)
    {
        context_ = context;
        readUsers(context_.getResourceAsStream("/xml/users.xml"));
    }


    /**
     * Must be called (e.g., by Login's init() method) once before
     * using any other method in class.
     *
     * @param context servlet's context object
     */
    public static synchronized void init(ServletContext context)
    {
        if (instance_ == null)
            instance_ = new UserManager(context);
    }


    /**
     * Adds a new user using the given username and password.
     * Returns the new User object if the operation succeeded or null if the
     * user could not be added because the username was already taken.
     *
     * @param username user's username
     * @param password user's password
     *
     * @return User object belonging to given username and password
     */
    public static synchronized User addUser(String username, String password)
    {
        return instance().addUserImpl(username, password);
    }


    /**
     * Adds a new user using the supplied username and password.
     * Returns the new User object if the operation succeeded or null if the
     * user could not be added becuase the username was already taken.
     *
     * @param username user's username
     * @param password user's password
     *
     * @return User object belonging to given username and password
     */
    private User addUserImpl(String username, String password)
    {
        // check to see if user already exists, returning immediately if so
        if (getUserImpl(username) != null)
            return null;

        // add user to in-memory database
        System.out.print("Adding \"" + username + "\" to main memory with " +
                         "password \"" + password + "\"...  ");
        User user = new User(username, password);
        users_.put(username, user);
        System.out.println("Done.");

        return user;
    }


    /**
     * Returns a User object for given username or null if one is not found.
     *
     * @param username user's username
     *
     * @return User object belonging to given username
     */
    public static synchronized User getUser(String username)
    {
        return instance().getUserImpl(username);
    }


    /**
     * Returns a User object for given username or null if one is not found.
     *
     * @param username user's username
     *
     * @return User object belonging to given username
     */
    private User getUserImpl(String username)
    {
        return (User)users_.get(username);
    }


    /**
     * Returns the one and only instance of this class, throwing an exception
     * if it has not been initialized.
     */
    private static UserManager instance()
    {
        if (instance_ == null)
            throw new RuntimeException("UserManager must be initialized " +
                                       "before use");
        return instance_;
    }


    /**
     * Read the user info from the input stream.
     *
     * @param in input stream
     */
    private void readUsers(InputStream in)
    {
        // try to parse users.xml, using internal UserReader class
        // as a ContentHandler, catching any errors
        try
        {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser parser = factory.newSAXParser();
            parser.parse(new InputSource(in), new UserReader());
        }
        catch (IOException e)
        {
            throw new RuntimeException("Error accessing user config file : " +
                                       e.getMessage());
        }
        catch (SAXException e)
        {
            throw new RuntimeException("Error parsing user config file: " +
                                       e.getMessage());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }


    /**
     * Saves the information about all users to the information database
     * (stored as a file).  Call this methods whenever you have made changed
     * to User objects you would like to save.  If you do not call this method,
     * your chages will be lost next time you start up the server.
     */
    public static synchronized void save()
    {
        instance().writeUsers();
    }


    /**
     * Writes users info to the config file, overwriting old contents.
     */
    private void writeUsers()
    {
        // try to write out users
        try
        {
            // prepare to pretty-print (manually)
            char [] LF  = new char[1];
            char [] TAB = new char[1];
            LF[0]  = '\n';
            TAB[0] = '\t';
            String path = context_.getRealPath("/xml/users.xml");
            System.out.println("Writing user info to : " + path);
            FileOutputStream fos = new FileOutputStream(path);
            Properties format = new Properties();
            format.put(OutputKeys.ENCODING, "UTF-8");
            format.put(OutputKeys.INDENT, "no");
            format.put(OutputKeys.METHOD, "xml");
            Serializer serializer = SerializerFactory.getSerializer(format);
            serializer.setOutputStream(fos);
            ContentHandler handler = serializer.asContentHandler();

            // start outputting
            handler.startDocument();
            AttributesImpl attributes = new AttributesImpl();
            handler.startElement(null, null, "users", attributes);
            handler.characters(LF, 0, 1);

            // iterate over the users and write them out
            Iterator iter = users_.keySet().iterator();
            while (iter.hasNext())
            {
                String username = (String)iter.next();
                User user = (User)users_.get(username);
                AttributesImpl atts = new AttributesImpl();
                atts.addAttribute(null, null, "name", null,
                                  user.getUserName());
                atts.addAttribute(null, null, "password", null,
                                  user.getPassword());
                handler.characters(TAB, 0, 1);
                handler.startElement(null, null, "user", atts);
                handler.characters(LF, 0, 1);

                // iterate over the user's news categories and write them out
                atts = new AttributesImpl();
                handler.characters(TAB, 0, 1);
                handler.characters(TAB, 0, 1);
                handler.startElement(null, null, "news-categories", atts);
                handler.characters(LF, 0, 1);
                Iterator cats = user.getNewsCategories().iterator();
                while (cats.hasNext())
                {
                    String cat = (String)cats.next();
                    AttributesImpl atts2 = new AttributesImpl();
                    atts2.addAttribute(null, null, "name", null, cat);
                    handler.characters(TAB, 0, 1);
                    handler.characters(TAB, 0, 1);
                    handler.characters(TAB, 0, 1);
                    handler.startElement(null, null, "category", atts2);
                    handler.endElement(null, null, "category");
                    handler.characters(LF, 0, 1);
                }
                handler.characters(TAB, 0, 1);
                handler.characters(TAB, 0, 1);
                handler.endElement(null, null, "news-categories");
                handler.characters(LF, 0, 1);
                handler.characters(TAB, 0, 1);
                handler.endElement(null, null, "user");
                handler.characters(LF, 0, 1);
            }

            // stop outputting
            handler.endElement(null, null, "users");
            handler.characters(LF, 0, 1);
            handler.endDocument();
        }

        // catch any errors
        catch (FileNotFoundException e)
        {
            throw new RuntimeException("Can't open file for writing:: " +
                                       e.getMessage());
        }
        catch (IOException e)
        {
            throw new RuntimeException("Can't open file for writing:: " +
                                       e.getMessage());
        }
        catch (SAXException e)
        {
            throw new RuntimeException("Can't write out users: " +
                                       e.getMessage());
        }
    }


    /**
     * An inner class which is the SAX2 handler used to read the user
     * XML file into Java objects.
     */
    private class UserReader extends DefaultHandler
    {
        // reference to user being parsed
        private User curUser_;


        /**
         * SAX handler for elements
         *
         * @param namespaceURI element's namespace (unused)
         * @param localName element's local name (unused)
         * @param qName element's name
         * @param atts element's attributes
         *
         * @throws SAXException
         */
        public void startElement(String namespaceURI,
                                 String localName,
                                 String qName,
                                 Attributes atts)
        throws SAXException
        {
            // process user elements
            if (qName.equals("user"))
            {
                String username = atts.getValue("name");
                if (username == null || username.equals(""))
                    throw new SAXException("user element must have a " +
                                           "name attribute");
                String password = atts.getValue("password");
                if (password == null || password.equals(""))
                    throw new SAXException("user element must have a " +
                                           "password attribute");
                curUser_ = addUserImpl(username, password);
                if (curUser_ == null)
                    throw new SAXException("Can't add user: " +
                                           "username " + username + " " +
                                           "is already taken");
            }

            // process category elements
            else if (qName.equals("category"))
            {
                String name = atts.getValue("name");
                if (name == null || name.equals(""))
                    throw new SAXException("category element must have a " +
                                           "name attribute");
                System.out.print("Associating \"" + name +
                                 "\" with \"" + curUser_.getUserName() +
                                 "\"...  ");
                curUser_.addNewsCategory(name);
                System.out.println("Done.");
            }
        }
    }

}
